package com.huimv.produce.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.produce.repo.ProdDayWeightEntity;
import com.huimv.produce.repo.ProdDayWeightRepo;
import com.huimv.produce.service.IWeight;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import javax.persistence.criteria.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class WeightImpl implements IWeight {
    @Autowired
    private ProdDayWeightRepo dayWeightRepo;

    /**
     * @Method      : getDayWeight
     * @Description : 
     * @Params      : [farmId, startDate, endDate]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/7       
     * @Time        : 13:58
     */
    @Override
    public Result getDayWeight(Integer farmId, String startDate, String endDate) {
        //
        List<ProdDayWeightEntity> dayWeightEntityList = dayWeightRepo.findByFarmIdAndStartAndEndDate(farmId, startDate, endDate);
        if (dayWeightEntityList.size() > 0) {
            BigDecimal totalBd = new BigDecimal(0);
            for (int a = 0; a < dayWeightEntityList.size(); a++) {
                ProdDayWeightEntity dayWeightEntity = dayWeightEntityList.get(a);
                totalBd = new BigDecimal(dayWeightEntity.getNetWeight()).add(totalBd);
            }
            JSONObject outJo = new JSONObject();
            outJo.put("total", totalBd.toString());
            outJo.put("weight", dayWeightEntityList);
            return new Result(ResultCode.SUCCESS, outJo);
        } else {
            return new Result(10001, "暂无数据.", false);
        }
    }

    /**
     * @Method      : getDayWeightLastNtimes
     * @Description : 
     * @Params      : [farmId, days]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/7       
     * @Time        : 13:58
     */
    @Override
    public Result getDayWeightLastNtimes(Integer farmId, Integer days) {
        //
        List<ProdDayWeightEntity> dayWeightEntityList = dayWeightRepo.findByFarmIdAndDays(farmId, days);
        if (dayWeightEntityList.size() > 0) {
            //排序
            List outList = new ArrayList();
            for(int a=dayWeightEntityList.size()-1;a>=0;a--){
                outList.add(dayWeightEntityList.get(a));
            }
            //计算总重
            BigDecimal totalBd = new BigDecimal(0);
            for (int a = 0; a < dayWeightEntityList.size(); a++) {
                ProdDayWeightEntity dayWeightEntity = dayWeightEntityList.get(a);
                totalBd = new BigDecimal(dayWeightEntity.getNetWeight()).add(totalBd);
            }
            JSONObject outJo = new JSONObject();
            outJo.put("total", totalBd.toString());
            outJo.put("weight", outList);
            return new Result(ResultCode.SUCCESS, outJo);
        } else {
            return new Result(10001, "暂无数据.", false);
        }
    }

    /**
     * @Method      : getWeightOnPage
     * @Description : 分页查询称重记录
     * @Params      : [farmId, startDate, endDate, pageNo, pageSize]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/7       
     * @Time        : 14:56
     */
    @Override
    public Result getWeightOnPage(Integer farmId, Date startDate, Date endDate, Integer pageNo, Integer pageSize) {
            Specification<ProdDayWeightEntity> sf = (Specification<ProdDayWeightEntity>) (root, criteriaQuery, criteriaBuilder) -> {
                //
                List<Predicate> predList = new ArrayList<>();
                if (null != farmId) {
                    predList.add(criteriaBuilder.equal(root.get("farmId").as(Integer.class), farmId));
                }
                if (null != startDate) {
                    predList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("addTime").as(Date.class), startDate));
                }
                if (null != endDate) {
                    predList.add(criteriaBuilder.lessThanOrEqualTo(root.get("addTime").as(Date.class), endDate));
                }
                //
                Predicate[] pred = new Predicate[predList.size()];
                Predicate and = criteriaBuilder.and(predList.toArray(pred));
                criteriaQuery.where(and);
                //
                List<Order> orders = new ArrayList<>();
                orders.add(criteriaBuilder.desc(root.get("id")));
                return criteriaQuery.orderBy(orders).getRestriction();
            };
            Pageable pageable = PageRequest.of( pageNo - 1, pageSize);
//        return new Result(ResultCode.SUCCESS,applyRepo.listApply(pageable,applyId));
            return new Result(ResultCode.SUCCESS, dayWeightRepo.findAll(sf, pageable));
    }
}
